var WebviewJsBridge = {
  _getCmd_: function(scheme, params) {
    params = this.stringify(params);
    var schemeTmp = this.stringify(scheme, null, '://');
    var cmd;

    if (this.isObject(scheme)) {
      cmd = schemeTmp;
    } else {
      cmd = 'mtcommand://' + schemeTmp;
    }
    cmd += (params ? ('?' + params) : '');
    console.log(cmd)
    return cmd;
  },
  _clearCall_: function(scheme, params) {
    var cmd = this._getCmd_(scheme, params);
    this.postMessage({
      cmd: cmd,
      clear: true
    });
    return this;
  },
  nativeCall: function(scheme, params, callback) {
    this._clearCall_(scheme, params);
    var cmd = this._getCmd_(scheme, params);
    var isFn = this.isFn;

    function receiveMessage(event) {
      var e = event.data;
      var receiveCmd = e.cmd;
      if (receiveCmd == cmd) {
        if (!e.clear) {
          callback.call(WebviewJsBridge, event.data);
        }
        window.removeEventListener('WebviewConnect', receiveMessage, false);
      }
    }
    // console.log(cmd)
    if (isFn(callback)) {
      window.addEventListener('WebviewConnect', receiveMessage, false);
    }

    location.href = cmd;

    return this;
  },
  postMessage: function(e) {
    this.dispatchEvent('WebviewConnect', e);
    return this;
  },
  stringify: function(obj, sep, eq) {
    if (!obj) return '';

    sep = sep || '&';
    eq = eq || '=';

    if (this.isObject(obj)) {
      return Object.keys(obj).map(function(k) {
        return k + eq + (this.isObject(obj[k]) ? JSON.stringify(obj[k]) : obj[k]);
      }.bind(this)).join(sep);
    } else {
      return obj;
    }
  },
  isObject: function(obj) {
    return Object.prototype.toString.call(obj) == '[object Object]';
  },
  isFn: function(fn) {
    return typeof fn === 'function';
  },
  dispatchEvent: function(eventname, data) {
    if (!this._eventList_)
      this._eventList_ = {};
    var eventInstance = this._eventList_[eventname];
    if (!eventInstance) {
      eventInstance = this._eventList_[eventname] = this._createEvent_(eventname);
    }
    if (data) {
      eventInstance.data = data;
    }
    window.dispatchEvent(eventInstance);
    return this;
  },
  _createEvent_: function(eventname) {
    if (this.isFn(document.CustomEvent)) {
      return new document.CustomEvent(eventname, {
        bubbles: true,
        cancelable: true
      });
    } else if (this.isFn(document.createEvent)) {
      var event = document.createEvent('Event');
      event.initEvent(eventname, true, true);
      return event;
    } else {
      return false;
    }
  },
  onSharePageInfo: function(args) {
    if (!this.isObject(args)) return this;
    this._setSharePageInfo_ = args;
    return this;
  },
  getSharePageInfo: function(args, setSharePageInfo) {
    if (this._sharePageInfo_) return this._sharePageInfo_;
    setSharePageInfo = this.isObject(setSharePageInfo) ? setSharePageInfo : {};

    var minWidth = 200;
    var minHeight = 200;
    var minByte = 10;

    if (this.isObject(args)) {
      if (args.width) {
        minWidth = parseInt(args.width, 10) || minWidth;
      }
      if (args.height) {
        minHeight = parseInt(args.height, 10) || minHeight;
      }
      if (args.byte) {
        minByte = parseInt(args.byte, 10) || minByte;
      }
    }
    var self = this;
    var sharePageInfo = this._sharePageInfo_ = {
      title: setSharePageInfo.title || getOgMetaContent('title') || document.title,
      image: setSharePageInfo.image || getOgMetaContent('image') || getShareImage(),
      description: setSharePageInfo.description || getOgMetaContent('description') || getShareDesc(),
      link: setSharePageInfo.link || getOgMetaContent('link') || location.href
    };

    function getOgMetaContent(flag) {
      var metaElement = document.querySelector('meta[property="og:' + flag + '"]');
      if (metaElement) {
        return metaElement.getAttribute('content');
      }
      return null;
    }

    function getShareImage() {
      var imgs = document.querySelectorAll('img');
      if (imgs.length) {
        for (var i = 0, len = imgs.length; i < len; i++) {
          var img = imgs[i];
          var width = img.width;
          var height = img.height;
          if (width >= minWidth && height >= minHeight) {
            return img.src;
          }
        }
      }
      return null;
    }

    function getShareDesc() {
      var descLabels = 'h1,h2,h3,h4,h5,p'.split(',');
      for (var i = 0, len = descLabels.length; i < len; i++) {
        var descLabel = document.querySelectorAll(descLabels[i]);
        if (descLabel.length) {
          for (var j = 0, length = descLabel.length; j < length; j++) {
            var label = descLabel[j];
            var txt = label.innerText;
            if (self.getByte(txt) >= minByte) {
              return txt;
            }
          }
        }
      }
      var descriptionMeta = document.querySelector('meta[name="description"]');
      if (descriptionMeta) {
        return descriptionMeta.getAttribute('content');
      }
      return null;
    }

    console.log('sharePageInfo => ', sharePageInfo);
    return sharePageInfo;
  },

  getByte: function(str) {
    str = str.toString().trim();
    var len = 0;
    for (var i = 0, j = str.length; i < j; i++) {
      if (str.charCodeAt(i) > 0 && str.charCodeAt(i) < 128) {
        len += 1;
      } else {
        len += 2;
      }
    }

    return Math.ceil(len / 2);
  },
  callSharePageInfo: function(args) { //客户端分享page
    var p = {};
    var setSharePageInfo = this._setSharePageInfo_ || {};
    var sharePageInfo = this.getSharePageInfo(args, setSharePageInfo);

    for (var key in sharePageInfo) {
      if (sharePageInfo.hasOwnProperty(key)) {
        if (setSharePageInfo[key]) {
          p[key] = encodeURIComponent(setSharePageInfo[key]);
        } else {
          p[key] = encodeURIComponent(sharePageInfo[key]);
        }
      }
    }

    // console.log('sharePageInfo => ', sharePageInfo);
    // console.log('setSharePageInfo => ', setSharePageInfo);
    var scheme = 'sharePageInfo';
    if (this.isFn(setSharePageInfo.success)) {
      this.nativeCall(scheme, p, function(e) {
        setSharePageInfo.success(e);
      });
    } else {
      this.nativeCall(scheme, p);
    }
  }
};
WebviewJsBridge.dispatchEvent('WebviewJsBridgeReady');
